@@ -59,7 +59,7 @@
 
 /* elementfactory information */
 static GstElementDetails gst_alsasink_details =
-GST_ELEMENT_DETAILS ("Audio Sink (ALSA)",
+GST_ELEMENT_DETAILS ("Audio sink (ALSA)",
     "Sink/Audio",
     "Output to a sound card via ALSA",
     "Wim Taymans <wim@fluendo.com>");
@@ -113,6 +113,12 @@ static GstStaticPadTemplate alsasink_sink_factory =
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
+        "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
+        "signed = (boolean) { TRUE, FALSE }, "
+        "width = (int) 32, "
+        "depth = (int) 32, "
+        "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 8 ]; "
+        "audio/x-raw-int, "
         "endianness = (int) { " ALSA_SINK_FACTORY_ENDIANNESS " }, "
         "signed = (boolean) { TRUE, FALSE }, "
         "width = (int) 16, "
@@ -166,6 +172,8 @@ gst_alsasink_finalise (GObject * object)
 
   g_free (sink->device);
   g_mutex_free (sink->alsa_lock);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static void
@@ -477,8 +485,8 @@ set_hwparams (GstAlsaSink * alsa)
 
   snd_pcm_hw_params_alloca (&params);
 
-  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz", alsa->channels,
-      alsa->rate);
+  GST_DEBUG_OBJECT (alsa, "Negotiating to %d channels @ %d Hz (format = %d)",
+      alsa->channels, alsa->rate, alsa->format);
 
   /* choose all parameters */
   CHECK (snd_pcm_hw_params_any (alsa->handle, params), no_config);
@@ -671,6 +679,10 @@ alsasink_parse_spec (GstAlsaSink * alsa, GstRingBufferSpec * spec)
 {
   switch (spec->type) {
     case GST_BUFTYPE_LINEAR:
+      GST_DEBUG_OBJECT (alsa,
+          "Linear format : depth=%d, width=%d, sign=%d, bigend=%d", spec->depth,
+          spec->width, spec->sign, spec->bigend);
+
       alsa->format = snd_pcm_build_linear_format (spec->depth, spec->width,
           spec->sign ? 0 : 1, spec->bigend ? 1 : 0);
       break;
