@@ -17,67 +17,40 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-theoradec
+ * @see_also: theoraenc, oggdemux
+ *
+ * <refsect2>
+ * <para>
+ * This element decodes theora streams into raw video
+ * <ulink url="http://www.theora.org/">Theora</ulink> is a royalty-free
+ * video codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
+ * Foundation</ulink>, based on the VP3 codec.
+ * </para>
+ * <para>
+ * </para>
+ * <title>Example pipeline</title>
+ * <programlisting>
+ * gst-launch -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! xvimagesink
+ * </programlisting>
+ * This example pipeline will decode an ogg stream and decodes the theora video. Refer to
+ * the theoraenc example to create the ogg file.
+ * </refsect2>
+ *
+ * Last reviewed on 2006-03-01 (0.10.4)
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#include <gst/gst.h>
-#include <theora/theora.h>
-#include <string.h>
+#include "gsttheoradec.h"
 #include <gst/tag/tag.h>
 
 GST_DEBUG_CATEGORY (theoradec_debug);
 #define GST_CAT_DEFAULT theoradec_debug
 
-#define GST_TYPE_THEORA_DEC \
-  (gst_theora_dec_get_type())
-#define GST_THEORA_DEC(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_THEORA_DEC,GstTheoraDec))
-#define GST_THEORA_DEC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_THEORA_DEC,GstTheoraDec))
-#define GST_IS_THEORA_DEC(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_THEORA_DEC))
-#define GST_IS_THEORA_DEC_CLASS(obj) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_THEORA_DEC))
-
-typedef struct _GstTheoraDec GstTheoraDec;
-typedef struct _GstTheoraDecClass GstTheoraDecClass;
-
-struct _GstTheoraDec
-{
-  GstElement element;
-
-  GstPad *sinkpad;
-  GstPad *srcpad;
-
-  theora_state state;
-  theora_info info;
-  theora_comment comment;
-
-  gboolean have_header;
-  guint64 granulepos;
-
-  GstClockTime last_timestamp;
-  guint64 frame_nr;
-  gboolean need_keyframe;
-  gint width, height;
-  gint offset_x, offset_y;
-
-  gboolean crop;
-
-  GList *queued;
-
-  gdouble segment_rate;
-  gint64 segment_start;
-  gint64 segment_stop;
-  gint64 segment_time;
-};
-
-struct _GstTheoraDecClass
-{
-  GstElementClass parent_class;
-};
-
 #define THEORA_DEF_CROP         TRUE
 enum
 {
@@ -85,13 +58,12 @@ enum
   ARG_CROP
 };
 
-static GstElementDetails theora_dec_details = {
-  "TheoraDec",
-  "Codec/Decoder/Video",
-  "decode raw theora streams to raw YUV video",
-  "Benjamin Otte <in7y118@public.uni-hamburg.de>, "
-      "Wim Taymans <wim@fluendo.com>",
-};
+static GstElementDetails theora_dec_details =
+GST_ELEMENT_DETAILS ("Theora video decoder",
+    "Codec/Decoder/Video",
+    "decode raw theora streams to raw YUV video",
+    "Benjamin Otte <in7y118@public.uni-hamburg.de>, "
+    "Wim Taymans <wim@fluendo.com>");
 
 static GstStaticPadTemplate theora_dec_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -130,7 +102,6 @@ static gboolean theora_dec_sink_convert (GstPad * pad,
     GstFormat src_format, gint64 src_value,
     GstFormat * dest_format, gint64 * dest_value);
 static gboolean theora_dec_sink_query (GstPad * pad, GstQuery * query);
-static GstCaps *theora_dec_src_getcaps (GstPad * pad);
 
 #if 0
 static const GstFormat *theora_get_formats (GstPad * pad);
@@ -184,10 +155,10 @@ gst_theora_dec_init (GstTheoraDec * dec, GstTheoraDecClass * g_class)
 
   dec->srcpad =
       gst_pad_new_from_static_template (&theora_dec_src_factory, "src");
-  gst_pad_set_getcaps_function (dec->srcpad, theora_dec_src_getcaps);
   gst_pad_set_event_function (dec->srcpad, theora_dec_src_event);
   gst_pad_set_query_type_function (dec->srcpad, theora_get_query_types);
   gst_pad_set_query_function (dec->srcpad, theora_dec_src_query);
+  gst_pad_use_fixed_caps (dec->srcpad);
 
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
@@ -195,6 +166,20 @@ gst_theora_dec_init (GstTheoraDec * dec, GstTheoraDecClass * g_class)
   dec->queued = NULL;
 }
 
+static void
+gst_theora_dec_reset (GstTheoraDec * dec)
+{
+  dec->need_keyframe = TRUE;
+  dec->last_timestamp = -1;
+  dec->granulepos = -1;
+  gst_segment_init (&dec->segment, GST_FORMAT_TIME);
+
+  GST_OBJECT_LOCK (dec);
+  dec->proportion = 1.0;
+  dec->earliest_time = -1;
+  GST_OBJECT_UNLOCK (dec);
+}
+
 /* FIXME: copy from libtheora, theora should somehow make this available for seeking */
 static int
 _theora_ilog (unsigned int v)
@@ -208,25 +193,54 @@ _theora_ilog (unsigned int v)
   return (ret);
 }
 
-static void
-_inc_granulepos (GstTheoraDec * dec)
+static gint64
+_theora_granule_frame (GstTheoraDec * dec, gint64 granulepos)
 {
   guint ilog;
   gint framecount;
 
-  if (dec->granulepos == -1)
-    return;
+  if (granulepos == -1)
+    return -1;
 
-  ilog = _theora_ilog (dec->info.keyframe_frequency_force - 1);
+  ilog = dec->granule_shift;
 
-  framecount = dec->granulepos >> ilog;
-  framecount += dec->granulepos - (framecount << ilog);
+  /* granulepos is last ilog bits for counting pframes since last iframe and 
+   * bits in front of that for the framenumber of the last iframe. */
+  framecount = granulepos >> ilog;
+  framecount += granulepos - (framecount << ilog);
 
   GST_DEBUG_OBJECT (dec, "framecount=%d, ilog=%u", framecount, ilog);
 
-  framecount++;
+  return framecount;
+}
 
-  dec->granulepos = (framecount << ilog);
+static GstClockTime
+_theora_granule_time (GstTheoraDec * dec, gint64 granulepos)
+{
+  gint framecount;
+
+  /* invalid granule results in invalid time */
+  if (granulepos == -1)
+    return -1;
+
+  /* get framecount */
+  framecount = _theora_granule_frame (dec, granulepos);
+
+  return gst_util_uint64_scale_int (framecount * GST_SECOND,
+      dec->info.fps_denominator, dec->info.fps_numerator);
+}
+
+static gint64
+_inc_granulepos (GstTheoraDec * dec, gint64 granulepos)
+{
+  gint framecount;
+
+  if (granulepos == -1)
+    return -1;
+
+  framecount = _theora_granule_frame (dec, granulepos);
+
+  return (framecount + 1) << dec->granule_shift;
 }
 
 #if 0
@@ -283,23 +297,23 @@ theora_dec_src_convert (GstPad * pad,
   GstTheoraDec *dec;
   guint64 scale = 1;
 
-  dec = GST_THEORA_DEC (GST_PAD_PARENT (pad));
-
-  /* we need the info part before we can done something */
-  if (!dec->have_header)
-    return FALSE;
-
   if (src_format == *dest_format) {
     *dest_value = src_value;
     return TRUE;
   }
 
+  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+
+  /* we need the info part before we can done something */
+  if (!dec->have_header)
+    goto no_header;
+
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
         case GST_FORMAT_DEFAULT:
-          *dest_value =
-              src_value * 2 / (dec->info.height * dec->info.width * 3);
+          *dest_value = gst_util_uint64_scale_int (src_value, 2,
+              dec->info.height * dec->info.width * 3);
           break;
         case GST_FORMAT_TIME:
           /* seems like a rather silly conversion, implement me if you like */
@@ -312,9 +326,8 @@ theora_dec_src_convert (GstPad * pad,
         case GST_FORMAT_BYTES:
           scale = 3 * (dec->info.width * dec->info.height) / 2;
         case GST_FORMAT_DEFAULT:
-          *dest_value =
-              scale * (((guint64) src_value * dec->info.fps_numerator) /
-              (dec->info.fps_denominator * GST_SECOND));
+          *dest_value = scale * gst_util_uint64_scale (src_value,
+              dec->info.fps_numerator, dec->info.fps_denominator * GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -323,12 +336,12 @@ theora_dec_src_convert (GstPad * pad,
     case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_TIME:
-          *dest_value = src_value * (GST_SECOND * dec->info.fps_denominator /
-              dec->info.fps_numerator);
+          *dest_value = gst_util_uint64_scale (src_value,
+              GST_SECOND * dec->info.fps_denominator, dec->info.fps_numerator);
           break;
         case GST_FORMAT_BYTES:
-          *dest_value =
-              src_value * 3 * (dec->info.width * dec->info.height) / 2;
+          *dest_value = gst_util_uint64_scale_int (src_value,
+              3 * dec->info.width * dec->info.height, 2);
           break;
         default:
           res = FALSE;
@@ -337,8 +350,17 @@ theora_dec_src_convert (GstPad * pad,
     default:
       res = FALSE;
   }
-
+done:
+  gst_object_unref (dec);
   return res;
+
+  /* ERRORS */
+no_header:
+  {
+    GST_DEBUG_OBJECT (dec, "no header yet, cannot convert");
+    res = FALSE;
+    goto done;
+  }
 }
 
 static gboolean
@@ -349,56 +371,41 @@ theora_dec_sink_convert (GstPad * pad,
   gboolean res = TRUE;
   GstTheoraDec *dec;
 
-  dec = GST_THEORA_DEC (GST_PAD_PARENT (pad));
-
-  /* we need the info part before we can done something */
-  if (!dec->have_header)
-    return FALSE;
-
   if (src_format == *dest_format) {
     *dest_value = src_value;
     return TRUE;
   }
 
+  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+
+  /* we need the info part before we can done something */
+  if (!dec->have_header)
+    goto no_header;
+
   switch (src_format) {
     case GST_FORMAT_DEFAULT:
-    {
-      guint64 framecount;
-      guint ilog;
-
-      ilog = _theora_ilog (dec->info.keyframe_frequency_force - 1);
-
-      /* granulepos is last ilog bits for counting pframes since last iframe and 
-       * bits in front of that for the framenumber of the last iframe. */
-      framecount = src_value >> ilog;
-      framecount += src_value - (framecount << ilog);
-
       switch (*dest_format) {
         case GST_FORMAT_TIME:
-          *dest_value = framecount * (GST_SECOND * dec->info.fps_denominator /
-              dec->info.fps_numerator);
+          *dest_value = _theora_granule_time (dec, src_value);
           break;
         default:
           res = FALSE;
       }
       break;
-    }
     case GST_FORMAT_TIME:
-    {
       switch (*dest_format) {
         case GST_FORMAT_DEFAULT:
         {
-          guint ilog = _theora_ilog (dec->info.keyframe_frequency_force - 1);
           guint rest;
 
           /* framecount */
-          *dest_value = src_value * dec->info.fps_numerator /
-              (GST_SECOND * dec->info.fps_denominator);
+          *dest_value = gst_util_uint64_scale (src_value,
+              dec->info.fps_numerator, GST_SECOND * dec->info.fps_denominator);
 
           /* funny way of calculating granulepos in theora */
           rest = *dest_value / dec->info.keyframe_frequency_force;
           *dest_value -= rest;
-          *dest_value <<= ilog;
+          *dest_value <<= dec->granule_shift;
           *dest_value += rest;
           break;
         }
@@ -407,20 +414,31 @@ theora_dec_sink_convert (GstPad * pad,
           break;
       }
       break;
-    }
     default:
       res = FALSE;
   }
-
+done:
+  gst_object_unref (dec);
   return res;
+
+  /* ERRORS */
+no_header:
+  {
+    GST_DEBUG_OBJECT (dec, "no header yet, cannot convert");
+    res = FALSE;
+    goto done;
+  }
 }
 
 static gboolean
 theora_dec_src_query (GstPad * pad, GstQuery * query)
 {
-  GstTheoraDec *dec = GST_THEORA_DEC (GST_PAD_PARENT (pad));
+  GstTheoraDec *dec;
+
   gboolean res = FALSE;
 
+  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
+
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:
     {
@@ -445,7 +463,7 @@ theora_dec_src_query (GstPad * pad, GstQuery * query)
                   granulepos, &my_format, &time)))
         goto error;
 
-      time = (time - dec->segment_start) + dec->segment_time;
+      time = (time - dec->segment.start) + dec->segment.time;
 
       GST_LOG_OBJECT (dec,
           "query %p: our time: %" GST_TIME_FORMAT, query, GST_TIME_ARGS (time));
@@ -481,14 +499,20 @@ theora_dec_src_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
+done:
+  gst_object_unref (dec);
+
   return res;
 
+  /* ERRORS */
 error:
-  GST_DEBUG ("query failed");
-  return res;
+  {
+    GST_DEBUG_OBJECT (dec, "query failed");
+    goto done;
+  }
 }
 
 static gboolean
@@ -512,7 +536,7 @@ theora_dec_sink_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
 
@@ -526,10 +550,11 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
   gboolean res = TRUE;
   GstTheoraDec *dec;
 
-  dec = GST_THEORA_DEC (GST_PAD_PARENT (pad));
+  dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_SEEK:{
+    case GST_EVENT_SEEK:
+    {
       GstFormat format, tformat;
       gdouble rate;
       GstEvent *real_seek;
@@ -540,6 +565,7 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
       gst_event_parse_seek (event, &rate, &format, &flags, &cur_type, &cur,
           &stop_type, &stop);
+      gst_event_unref (event);
 
       /* we have to ask our peer to seek to time here as we know
        * nothing about how to generate a granulepos from the src
@@ -549,9 +575,9 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
        */
       tformat = GST_FORMAT_TIME;
       if (!(res = theora_dec_src_convert (pad, format, cur, &tformat, &tcur)))
-        goto error;
+        goto convert_error;
       if (!(res = theora_dec_src_convert (pad, format, stop, &tformat, &tstop)))
-        goto error;
+        goto convert_error;
 
       /* then seek with time on the peer */
       real_seek = gst_event_new_seek (rate, GST_FORMAT_TIME,
@@ -559,33 +585,46 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
       res = gst_pad_push_event (dec->sinkpad, real_seek);
 
-      gst_event_unref (event);
+      break;
+    }
+    case GST_EVENT_QOS:
+    {
+      gdouble proportion;
+      GstClockTimeDiff diff;
+      GstClockTime timestamp;
+
+      gst_event_parse_qos (event, &proportion, &diff, &timestamp);
+
+      /* we cannot randomly skip frame decoding since we don't have
+       * B frames. we can however use the timestamp and diff to not
+       * push late frames. This would at least save us the time to
+       * crop/memcpy the data. */
+      GST_OBJECT_LOCK (dec);
+      dec->proportion = proportion;
+      dec->earliest_time = timestamp + diff;
+      GST_OBJECT_UNLOCK (dec);
+
+      GST_DEBUG_OBJECT (dec, "got QoS %" GST_TIME_FORMAT ", %" G_GINT64_FORMAT,
+          GST_TIME_ARGS (timestamp), diff);
+
+      res = gst_pad_push_event (dec->sinkpad, event);
       break;
     }
     default:
-      res = gst_pad_event_default (pad, event);
+      res = gst_pad_push_event (dec->sinkpad, event);
       break;
   }
+done:
+  gst_object_unref (dec);
 
   return res;
 
-error:
-  gst_event_unref (event);
-  return res;
-}
-
-static GstCaps *
-theora_dec_src_getcaps (GstPad * pad)
-{
-  GstCaps *caps;
-
-  GST_OBJECT_LOCK (pad);
-  if (!(caps = GST_PAD_CAPS (pad)))
-    caps = (GstCaps *) gst_pad_get_pad_template_caps (pad);
-  caps = gst_caps_ref (caps);
-  GST_OBJECT_UNLOCK (pad);
-
-  return caps;
+  /* ERRORS */
+convert_error:
+  {
+    GST_DEBUG_OBJECT (dec, "could not convert format");
+    goto done;
+  }
 }
 
 static gboolean
@@ -598,17 +637,26 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
 
   GST_LOG_OBJECT (dec, "handling event");
   switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FLUSH_START:
+      ret = gst_pad_push_event (dec->srcpad, event);
+      break;
+    case GST_EVENT_FLUSH_STOP:
+      /* FIXME, makes us waiting for keyframes */
+      gst_theora_dec_reset (dec);
+      ret = gst_pad_push_event (dec->srcpad, event);
+      break;
     case GST_EVENT_EOS:
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
     case GST_EVENT_NEWSEGMENT:
     {
+      gboolean update;
       GstFormat format;
       gdouble rate;
       gint64 start, stop, time;
 
-      gst_event_parse_new_segment (event, NULL, &rate, &format, &start, &stop,
-          &time);
+      gst_event_parse_new_segment (event, &update, &rate, &format, &start,
+          &stop, &time);
 
       /* we need TIME and a positive rate */
       if (format != GST_FORMAT_TIME)
@@ -617,15 +665,11 @@ theora_dec_sink_event (GstPad * pad, GstEvent * event)
       if (rate <= 0.0)
         goto newseg_wrong_rate;
 
-      /* now copy over the values */
-      dec->segment_rate = rate;
-      dec->segment_start = start;
-      dec->segment_stop = stop;
-      dec->segment_time = time;
+      /* now configure the values */
+      gst_segment_set_newsegment (&dec->segment, update,
+          rate, format, start, stop, time);
 
-      dec->need_keyframe = TRUE;
-      dec->granulepos = -1;
-      dec->last_timestamp = -1;
+      /* and forward */
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
     }
@@ -641,12 +685,14 @@ done:
   /* ERRORS */
 newseg_wrong_format:
   {
-    GST_DEBUG ("received non TIME newsegment");
+    GST_DEBUG_OBJECT (dec, "received non TIME newsegment");
+    gst_event_unref (event);
     goto done;
   }
 newseg_wrong_rate:
   {
-    GST_DEBUG ("negative rates not supported yet");
+    GST_DEBUG_OBJECT (dec, "negative rates not supported yet");
+    gst_event_unref (event);
     goto done;
   }
 }
@@ -662,7 +708,7 @@ theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
   GstBuffer *buf;
   GstTagList *list;
 
-  GST_DEBUG ("parsing comment packet");
+  GST_DEBUG_OBJECT (dec, "parsing comment packet");
 
   buf = gst_buffer_new_and_alloc (packet->bytes);
   memcpy (GST_BUFFER_DATA (buf), packet->packet, packet->bytes);
@@ -743,6 +789,8 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
     dec->offset_y = 0;
   }
 
+  dec->granule_shift = _theora_ilog (dec->info.keyframe_frequency_force - 1);
+
   GST_DEBUG_OBJECT (dec, "after fixup frame dimension %dx%d, offset %d:%d",
       dec->width, dec->height, dec->offset_x, dec->offset_y);
 
@@ -768,7 +816,7 @@ theora_handle_header_packet (GstTheoraDec * dec, ogg_packet * packet)
 {
   GstFlowReturn res;
 
-  GST_DEBUG ("parsing header packet");
+  GST_DEBUG_OBJECT (dec, "parsing header packet");
 
   if (theora_decode_header (&dec->info, &dec->comment, packet))
     goto header_read_error;
@@ -799,6 +847,7 @@ header_read_error:
   }
 }
 
+/* FIXME, this needs to be moved to the demuxer */
 static GstFlowReturn
 theora_dec_push (GstTheoraDec * dec, GstBuffer * buf)
 {
@@ -822,12 +871,12 @@ theora_dec_push (GstTheoraDec * dec, GstBuffer * buf)
         GstBuffer *buffer = GST_BUFFER (walk->data);
         GstClockTime time;
 
-        time = outtime - ((size * GST_SECOND * dec->info.fps_denominator)
-            / dec->info.fps_numerator);
+        time = outtime - gst_util_uint64_scale_int (size * GST_SECOND,
+            dec->info.fps_denominator, dec->info.fps_numerator);
 
         GST_DEBUG_OBJECT (dec, "patch buffer %lld %lld", size, time);
         GST_BUFFER_TIMESTAMP (buffer) = time;
-        /* ignore the result */
+        /* ignore the result.. */
         gst_pad_push (dec->srcpad, buffer);
         size--;
       }
@@ -867,16 +916,38 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
     goto dropping;
   }
 
+  /* this does the decoding */
   if (theora_decode_packetin (&dec->state, packet))
-    goto could_not_read;
-
-  if (theora_decode_YUVout (&dec->state, &yuv) < 0)
     goto decode_error;
 
+  if (outtime != -1) {
+    gboolean need_skip;
+    GstClockTime qostime;
+
+    /* qos needs to be done on running time */
+    qostime = gst_segment_to_running_time (&dec->segment, GST_FORMAT_TIME,
+        outtime);
+
+    GST_OBJECT_LOCK (dec);
+    /* check for QoS, don't perform the last steps of getting and
+     * pushing the buffers that are known to be late. */
+    /* FIXME, we can also entirely skip decoding if the next valid buffer is 
+     * known to be after a keyframe (using the granule_shift) */
+    need_skip = dec->earliest_time != -1 && qostime <= dec->earliest_time;
+    GST_OBJECT_UNLOCK (dec);
+
+    if (need_skip)
+      goto dropping_qos;
+  }
+
+  /* this does postprocessing and set up the decoded frame
+   * pointers in our yuv variable */
+  if (theora_decode_YUVout (&dec->state, &yuv) < 0)
+    goto no_yuv;
+
   if ((yuv.y_width != dec->info.width) || (yuv.y_height != dec->info.height))
     goto wrong_dimensions;
 
-
   width = dec->width;
   height = dec->height;
   cwidth = width / 2;
@@ -909,6 +980,7 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
     guchar *dest_y, *src_y;
     guchar *dest_u, *src_u;
     guchar *dest_v, *src_v;
+    gint offset;
 
     dest_y = GST_BUFFER_DATA (out);
     dest_u = dest_y + stride_y * height;
@@ -923,8 +995,10 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
       src_y += yuv.y_stride;
     }
 
-    src_u = yuv.u + dec->offset_x / 2 + dec->offset_y / 2 * yuv.uv_stride;
-    src_v = yuv.v + dec->offset_x / 2 + dec->offset_y / 2 * yuv.uv_stride;
+    offset = dec->offset_x / 2 + dec->offset_y / 2 * yuv.uv_stride;
+
+    src_u = yuv.u + offset;
+    src_v = yuv.v + offset;
 
     for (i = 0; i < cheight; i++) {
       memcpy (dest_u, src_u, cwidth);
@@ -937,12 +1011,13 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
     }
   }
 
+  /* FIXME, frame_nr not correct */
   GST_BUFFER_OFFSET (out) = dec->frame_nr;
   dec->frame_nr++;
   GST_BUFFER_OFFSET_END (out) = dec->frame_nr;
   GST_BUFFER_DURATION (out) =
-      GST_SECOND * ((gdouble) dec->info.fps_denominator) /
-      dec->info.fps_numerator;
+      gst_util_uint64_scale_int (GST_SECOND, dec->info.fps_denominator,
+      dec->info.fps_numerator);
   GST_BUFFER_TIMESTAMP (out) = outtime;
 
   result = theora_dec_push (dec, out);
@@ -961,13 +1036,19 @@ dropping:
     GST_WARNING_OBJECT (dec, "dropping frame because we need a keyframe");
     return GST_FLOW_OK;
   }
-could_not_read:
+dropping_qos:
   {
-    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
-        (NULL), ("theora decoder did not read data packet"));
-    return GST_FLOW_ERROR;
+    dec->frame_nr++;
+    GST_WARNING_OBJECT (dec, "dropping frame because of QoS");
+    return GST_FLOW_OK;
   }
 decode_error:
+  {
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+        (NULL), ("theora decoder did not decode data packet"));
+    return GST_FLOW_ERROR;
+  }
+no_yuv:
   {
     GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
         (NULL), ("couldn't read out YUV image"));
@@ -975,12 +1056,14 @@ decode_error:
   }
 wrong_dimensions:
   {
-    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, DECODE,
+    GST_ELEMENT_ERROR (GST_ELEMENT (dec), STREAM, FORMAT,
         (NULL), ("dimensions of image do not match header"));
     return GST_FLOW_ERROR;
   }
 no_buffer:
   {
+    GST_DEBUG_OBJECT (dec, "could not get buffer, reason: %s",
+        gst_flow_get_name (result));
     return result;
   }
 }
@@ -994,27 +1077,36 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
 
   dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
 
+  /* resync on DISCONT */
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT))) {
+    dec->need_keyframe = TRUE;
+    dec->last_timestamp = -1;
+    dec->granulepos = -1;
+  }
+
   /* make ogg_packet out of the buffer */
   packet.packet = GST_BUFFER_DATA (buf);
   packet.bytes = GST_BUFFER_SIZE (buf);
   packet.granulepos = GST_BUFFER_OFFSET_END (buf);
   packet.packetno = 0;          /* we don't really care */
   packet.b_o_s = dec->have_header ? 0 : 1;
+  /* EOS does not matter for the decoder */
   packet.e_o_s = 0;
 
   if (dec->have_header) {
     if (packet.granulepos != -1) {
       dec->granulepos = packet.granulepos;
-      dec->last_timestamp =
-          GST_SECOND * theora_granule_time (&dec->state, packet.granulepos);
+      dec->last_timestamp = _theora_granule_time (dec, packet.granulepos);
     } else if (dec->last_timestamp != -1) {
-      dec->last_timestamp +=
-          (GST_SECOND * dec->info.fps_denominator) / dec->info.fps_numerator;
+      dec->last_timestamp = _theora_granule_time (dec, dec->granulepos);
     }
   } else {
     dec->last_timestamp = -1;
   }
 
+  if (packet.bytes < 1)
+    goto wrong_size;
+
   GST_DEBUG_OBJECT (dec, "header=%d packetno=%lld, outtime=%" GST_TIME_FORMAT,
       packet.packet[0], packet.packetno, GST_TIME_ARGS (dec->last_timestamp));
 
@@ -1030,13 +1122,22 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
   }
 
 done:
-  _inc_granulepos (dec);
+  /* interpollate granule pos */
+  dec->granulepos = _inc_granulepos (dec, dec->granulepos);
 
   gst_object_unref (dec);
 
   gst_buffer_unref (buf);
 
   return result;
+
+  /* ERRORS */
+wrong_size:
+  {
+    GST_WARNING_OBJECT (dec, "received empty packet");
+    result = GST_FLOW_OK;
+    goto done;
+  }
 }
 
 static GstStateChangeReturn
@@ -1053,9 +1154,7 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
       theora_info_init (&dec->info);
       theora_comment_init (&dec->comment);
       dec->have_header = FALSE;
-      dec->need_keyframe = TRUE;
-      dec->last_timestamp = -1;
-      dec->granulepos = -1;
+      gst_theora_dec_reset (dec);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
@@ -1072,8 +1171,6 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
       theora_clear (&dec->state);
       theora_comment_clear (&dec->comment);
       theora_info_clear (&dec->info);
-      dec->have_header = FALSE;
-      dec->granulepos = -1;
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
