@@ -20,30 +20,39 @@
 
 /**
  * SECTION:element-vorbisparse
+ * @short_description: parses vorbis streams 
  * @see_also: vorbisdec, oggdemux, theoraparse
  *
+ * <refsect2>
+ * <para>
  * The vorbisparse element will parse the header packets of the Vorbis
  * stream and put them as the streamheader in the caps. This is used in the
  * multifdsink case where you want to stream live vorbis streams to multiple
  * clients, each client has to receive the streamheaders first before they can
  * consume the vorbis packets.
- *
+ * </para>
+ * <para>
  * This element also makes sure that the buffers that it pushes out are properly
  * timestamped and that their offset and offset_end are set. The buffers that
  * vorbisparse outputs have all of the metadata that oggmux expects to receive,
  * which allows you to (for example) remux an ogg/vorbis file.
- *
- * <refsect2>
+ * </para>
  * <title>Example pipelines</title>
- * |[
+ * <para>
+ * <programlisting>
  * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisparse ! fakesink
- * ]| This pipeline shows that the streamheader is set in the caps, and that each
+ * </programlisting>
+ * This pipeline shows that the streamheader is set in the caps, and that each
  * buffer has the timestamp, duration, offset, and offset_end set.
- * |[
+ * </para>
+ * <para>
+ * <programlisting>
  * gst-launch filesrc location=sine.ogg ! oggdemux ! vorbisparse \
  *            ! oggmux ! filesink location=sine-remuxed.ogg
- * ]| This pipeline shows remuxing. sine-remuxed.ogg might not be exactly the same
+ * </programlisting>
+ * This pipeline shows remuxing. sine-remuxed.ogg might not be exactly the same
  * as sine.ogg, but they should produce exactly the same decoded data.
+ * </para>
  * </refsect2>
  *
  * Last reviewed on 2006-04-01 (0.10.4.1)
@@ -58,13 +67,25 @@
 GST_DEBUG_CATEGORY_EXTERN (vorbisparse_debug);
 #define GST_CAT_DEFAULT vorbisparse_debug
 
-static const GstElementDetails vorbis_parse_details = {
+static GstElementDetails vorbis_parse_details = {
   "VorbisParse",
   "Codec/Parser/Audio",
   "parse raw vorbis streams",
   "Thomas Vander Stichele <thomas at apestaart dot org>"
 };
 
+/* Filter signals and args */
+enum
+{
+  /* FILL ME */
+  LAST_SIGNAL
+};
+
+enum
+{
+  ARG_0
+};
+
 static GstStaticPadTemplate vorbis_parse_sink_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -86,12 +107,6 @@ static GstFlowReturn vorbis_parse_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn vorbis_parse_change_state (GstElement * element,
     GstStateChange transition);
 static gboolean vorbis_parse_sink_event (GstPad * pad, GstEvent * event);
-static gboolean vorbis_parse_src_query (GstPad * pad, GstQuery * query);
-static gboolean vorbis_parse_convert (GstPad * pad,
-    GstFormat src_format, gint64 src_value,
-    GstFormat * dest_format, gint64 * dest_value);
-static GstFlowReturn vorbis_parse_parse_packet (GstVorbisParse * parse,
-    GstBuffer * buf);
 
 static void
 gst_vorbis_parse_base_init (gpointer g_class)
@@ -111,8 +126,6 @@ gst_vorbis_parse_class_init (GstVorbisParseClass * klass)
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
   gstelement_class->change_state = vorbis_parse_change_state;
-
-  klass->parse_packet = GST_DEBUG_FUNCPTR (vorbis_parse_parse_packet);
 }
 
 static void
@@ -120,16 +133,12 @@ gst_vorbis_parse_init (GstVorbisParse * parse, GstVorbisParseClass * g_class)
 {
   parse->sinkpad =
       gst_pad_new_from_static_template (&vorbis_parse_sink_factory, "sink");
-  gst_pad_set_chain_function (parse->sinkpad,
-      GST_DEBUG_FUNCPTR (vorbis_parse_chain));
-  gst_pad_set_event_function (parse->sinkpad,
-      GST_DEBUG_FUNCPTR (vorbis_parse_sink_event));
+  gst_pad_set_chain_function (parse->sinkpad, vorbis_parse_chain);
+  gst_pad_set_event_function (parse->sinkpad, vorbis_parse_sink_event);
   gst_element_add_pad (GST_ELEMENT (parse), parse->sinkpad);
 
   parse->srcpad =
       gst_pad_new_from_static_template (&vorbis_parse_src_factory, "src");
-  gst_pad_set_query_function (parse->srcpad,
-      GST_DEBUG_FUNCPTR (vorbis_parse_src_query));
   gst_element_add_pad (GST_ELEMENT (parse), parse->srcpad);
 }
 
@@ -177,23 +186,12 @@ vorbis_parse_set_header_on_caps (GstVorbisParse * parse, GstCaps * caps)
   g_value_unset (&array);
 }
 
-static void
-vorbis_parse_drain_event_queue (GstVorbisParse * parse)
-{
-  while (parse->event_queue->length) {
-    GstEvent *event;
-
-    event = GST_EVENT_CAST (g_queue_pop_head (parse->event_queue));
-    gst_pad_event_default (parse->sinkpad, event);
-  }
-}
-
 static void
 vorbis_parse_push_headers (GstVorbisParse * parse)
 {
   /* mark and put on caps */
   GstCaps *caps;
-  GstBuffer *outbuf, *outbuf1, *outbuf2, *outbuf3;
+  GstBuffer *outbuf;
   ogg_packet packet;
 
   /* get the headers into the caps, passing them to vorbis as we go */
@@ -203,16 +201,18 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   gst_pad_set_caps (parse->srcpad, caps);
   gst_caps_unref (caps);
 
+  /* push out buffers, ignoring return value... */
   outbuf = GST_BUFFER_CAST (parse->streamheader->data);
   packet.packet = GST_BUFFER_DATA (outbuf);
   packet.bytes = GST_BUFFER_SIZE (outbuf);
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 1;
   packet.e_o_s = 0;
-  packet.b_o_s = 1;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
   parse->sample_rate = parse->vi.rate;
-  outbuf1 = outbuf;
+
+  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (parse->srcpad));
+  gst_pad_push (parse->srcpad, outbuf);
 
   outbuf = GST_BUFFER_CAST (parse->streamheader->next->data);
   packet.packet = GST_BUFFER_DATA (outbuf);
@@ -220,9 +220,10 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 2;
   packet.e_o_s = 0;
-  packet.b_o_s = 0;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
-  outbuf2 = outbuf;
+
+  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (parse->srcpad));
+  gst_pad_push (parse->srcpad, outbuf);
 
   outbuf = GST_BUFFER_CAST (parse->streamheader->next->next->data);
   packet.packet = GST_BUFFER_DATA (outbuf);
@@ -230,20 +231,10 @@ vorbis_parse_push_headers (GstVorbisParse * parse)
   packet.granulepos = GST_BUFFER_OFFSET_END (outbuf);
   packet.packetno = 3;
   packet.e_o_s = 0;
-  packet.b_o_s = 0;
   vorbis_synthesis_headerin (&parse->vi, &parse->vc, &packet);
-  outbuf3 = outbuf;
 
-  /* first process queued events */
-  vorbis_parse_drain_event_queue (parse);
-
-  /* push out buffers, ignoring return value... */
-  gst_buffer_set_caps (outbuf1, GST_PAD_CAPS (parse->srcpad));
-  gst_pad_push (parse->srcpad, outbuf1);
-  gst_buffer_set_caps (outbuf2, GST_PAD_CAPS (parse->srcpad));
-  gst_pad_push (parse->srcpad, outbuf2);
-  gst_buffer_set_caps (outbuf3, GST_PAD_CAPS (parse->srcpad));
-  gst_pad_push (parse->srcpad, outbuf3);
+  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (parse->srcpad));
+  gst_pad_push (parse->srcpad, outbuf);
 
   g_list_free (parse->streamheader);
   parse->streamheader = NULL;
@@ -260,12 +251,6 @@ vorbis_parse_clear_queue (GstVorbisParse * parse)
     buf = GST_BUFFER_CAST (g_queue_pop_head (parse->buffer_queue));
     gst_buffer_unref (buf);
   }
-  while (parse->event_queue->length) {
-    GstEvent *event;
-
-    event = GST_EVENT_CAST (g_queue_pop_head (parse->event_queue));
-    gst_event_unref (event);
-  }
 }
 
 static GstFlowReturn
@@ -299,10 +284,6 @@ vorbis_parse_drain_queue_prematurely (GstVorbisParse * parse)
    * didn't-get-a-granulepos-on-the-last-packet case. Assuming a continuous
    * stream. */
 
-  /* if we got EOS before any buffers came, go ahead and push the other events
-   * first */
-  vorbis_parse_drain_event_queue (parse);
-
   while (!g_queue_is_empty (parse->buffer_queue)) {
     GstBuffer *buf;
 
@@ -390,10 +371,15 @@ vorbis_parse_queue_buffer (GstVorbisParse * parse, GstBuffer * buf)
 }
 
 static GstFlowReturn
-vorbis_parse_parse_packet (GstVorbisParse * parse, GstBuffer * buf)
+vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstFlowReturn ret;
+  GstBuffer *buf;
+  GstVorbisParse *parse;
 
+  parse = GST_VORBIS_PARSE (gst_pad_get_parent (pad));
+
+  buf = GST_BUFFER (buffer);
   parse->packetno++;
 
   if (parse->packetno <= 3) {
@@ -408,29 +394,7 @@ vorbis_parse_parse_packet (GstVorbisParse * parse, GstBuffer * buf)
     ret = vorbis_parse_queue_buffer (parse, buf);
   }
 
-  return ret;
-}
-
-static GstFlowReturn
-vorbis_parse_chain (GstPad * pad, GstBuffer * buffer)
-{
-  GstVorbisParseClass *klass;
-  GstVorbisParse *parse;
-
-  parse = GST_VORBIS_PARSE (GST_PAD_PARENT (pad));
-  klass = GST_VORBIS_PARSE_CLASS (G_OBJECT_GET_CLASS (parse));
-
-  g_assert (klass->parse_packet != NULL);
-
-  return klass->parse_packet (parse, buffer);
-}
-
-static gboolean
-vorbis_parse_queue_event (GstVorbisParse * parse, GstEvent * event)
-{
-  GstFlowReturn ret = TRUE;
-
-  g_queue_push_tail (parse->event_queue, event);
+  gst_object_unref (parse);
 
   return ret;
 }
@@ -455,10 +419,7 @@ vorbis_parse_sink_event (GstPad * pad, GstEvent * event)
       ret = gst_pad_event_default (pad, event);
       break;
     default:
-      if (!parse->streamheader_sent && GST_EVENT_IS_SERIALIZED (event))
-        ret = vorbis_parse_queue_event (parse, event);
-      else
-        ret = gst_pad_event_default (pad, event);
+      ret = gst_pad_event_default (pad, event);
       break;
   }
 
@@ -467,155 +428,6 @@ vorbis_parse_sink_event (GstPad * pad, GstEvent * event)
   return ret;
 }
 
-static gboolean
-vorbis_parse_convert (GstPad * pad,
-    GstFormat src_format, gint64 src_value,
-    GstFormat * dest_format, gint64 * dest_value)
-{
-  gboolean res = TRUE;
-  GstVorbisParse *parse;
-  guint64 scale = 1;
-
-  parse = GST_VORBIS_PARSE (GST_PAD_PARENT (pad));
-
-  /* fixme: assumes atomic access to lots of instance variables modified from
-   * the streaming thread, including 64-bit variables */
-
-  if (parse->packetno < 4)
-    return FALSE;
-
-  if (src_format == *dest_format) {
-    *dest_value = src_value;
-    return TRUE;
-  }
-
-  if (parse->sinkpad == pad &&
-      (src_format == GST_FORMAT_BYTES || *dest_format == GST_FORMAT_BYTES))
-    return FALSE;
-
-  switch (src_format) {
-    case GST_FORMAT_TIME:
-      switch (*dest_format) {
-        case GST_FORMAT_BYTES:
-          scale = sizeof (float) * parse->vi.channels;
-        case GST_FORMAT_DEFAULT:
-          *dest_value =
-              scale * gst_util_uint64_scale_int (src_value, parse->vi.rate,
-              GST_SECOND);
-          break;
-        default:
-          res = FALSE;
-      }
-      break;
-    case GST_FORMAT_DEFAULT:
-      switch (*dest_format) {
-        case GST_FORMAT_BYTES:
-          *dest_value = src_value * sizeof (float) * parse->vi.channels;
-          break;
-        case GST_FORMAT_TIME:
-          *dest_value =
-              gst_util_uint64_scale_int (src_value, GST_SECOND, parse->vi.rate);
-          break;
-        default:
-          res = FALSE;
-      }
-      break;
-    case GST_FORMAT_BYTES:
-      switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_value = src_value / (sizeof (float) * parse->vi.channels);
-          break;
-        case GST_FORMAT_TIME:
-          *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
-              parse->vi.rate * sizeof (float) * parse->vi.channels);
-          break;
-        default:
-          res = FALSE;
-      }
-      break;
-    default:
-      res = FALSE;
-  }
-
-  return res;
-}
-
-static gboolean
-vorbis_parse_src_query (GstPad * pad, GstQuery * query)
-{
-  gint64 granulepos;
-  GstVorbisParse *parse;
-  gboolean res = FALSE;
-
-  parse = GST_VORBIS_PARSE (GST_PAD_PARENT (pad));
-
-  switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_POSITION:
-    {
-      GstFormat format;
-      gint64 value;
-
-      granulepos = parse->prev_granulepos;
-
-      gst_query_parse_position (query, &format, NULL);
-
-      /* and convert to the final format */
-      if (!(res =
-              vorbis_parse_convert (pad, GST_FORMAT_DEFAULT, granulepos,
-                  &format, &value)))
-        goto error;
-
-      /* fixme: support segments
-         value = (value - parse->segment_start) + parse->segment_time;
-       */
-
-      gst_query_set_position (query, format, value);
-
-      GST_LOG_OBJECT (parse, "query %p: peer returned granulepos: %"
-          G_GUINT64_FORMAT " - we return %" G_GUINT64_FORMAT " (format %u)",
-          query, granulepos, value, format);
-
-      break;
-    }
-    case GST_QUERY_DURATION:
-    {
-      /* fixme: not threadsafe */
-      /* query peer for total length */
-      if (!gst_pad_is_linked (parse->sinkpad)) {
-        GST_WARNING_OBJECT (parse, "sink pad %" GST_PTR_FORMAT " is not linked",
-            parse->sinkpad);
-        goto error;
-      }
-      if (!(res = gst_pad_query (GST_PAD_PEER (parse->sinkpad), query)))
-        goto error;
-      break;
-    }
-    case GST_QUERY_CONVERT:
-    {
-      GstFormat src_fmt, dest_fmt;
-      gint64 src_val, dest_val;
-
-      gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
-      if (!(res =
-              vorbis_parse_convert (pad, src_fmt, src_val, &dest_fmt,
-                  &dest_val)))
-        goto error;
-      gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
-      break;
-    }
-    default:
-      res = gst_pad_query_default (pad, query);
-      break;
-  }
-  return res;
-
-error:
-  {
-    GST_WARNING_OBJECT (parse, "error handling query");
-    return res;
-  }
-}
-
 static GstStateChangeReturn
 vorbis_parse_change_state (GstElement * element, GstStateChange transition)
 {
@@ -631,7 +443,6 @@ vorbis_parse_change_state (GstElement * element, GstStateChange transition)
       parse->packetno = 0;
       parse->streamheader_sent = FALSE;
       parse->buffer_queue = g_queue_new ();
-      parse->event_queue = g_queue_new ();
       break;
     default:
       break;
@@ -646,8 +457,6 @@ vorbis_parse_change_state (GstElement * element, GstStateChange transition)
       vorbis_parse_clear_queue (parse);
       g_queue_free (parse->buffer_queue);
       parse->buffer_queue = NULL;
-      g_queue_free (parse->event_queue);
-      parse->event_queue = NULL;
       break;
     default:
       break;
