@@ -30,11 +30,6 @@
 
 #include <string.h>
 #include <stdlib.h>
-#include <math.h>
-
-#ifndef M_PI
-#define M_PI  3.14159265358979323846
-#endif
 
 static unsigned char
 random_char (void)
@@ -209,31 +204,31 @@ enum
   COLOR_DARK_GREY
 };
 
-static const struct vts_color_struct vts_colors[] = {
+static struct vts_color_struct vts_colors[] = {
   /* 100% white */
-  {255, 128, 128, 255, 255, 255, 255},
+  {255, 128, 128, 255, 255, 255},
   /* yellow */
-  {226, 0, 155, 255, 255, 0, 255},
+  {226, 0, 155, 255, 255, 0},
   /* cyan */
-  {179, 170, 0, 0, 255, 255, 255},
+  {179, 170, 0, 0, 255, 255},
   /* green */
-  {150, 46, 21, 0, 255, 0, 255},
+  {150, 46, 21, 0, 255, 0},
   /* magenta */
-  {105, 212, 235, 255, 0, 255, 255},
+  {105, 212, 235, 255, 0, 255},
   /* red */
-  {76, 85, 255, 255, 0, 0, 255},
+  {76, 85, 255, 255, 0, 0},
   /* blue */
-  {29, 255, 107, 0, 0, 255, 255},
+  {29, 255, 107, 0, 0, 255},
   /* black */
-  {16, 128, 128, 0, 0, 0, 255},
+  {16, 128, 128, 0, 0, 0},
   /* -I */
-  {16, 198, 21, 0, 0, 128, 255},
+  {16, 198, 21, 0, 0, 128},
   /* +Q */
-  {16, 235, 198, 0, 128, 255, 255},
+  {16, 235, 198, 0, 128, 255},
   /* superblack */
-  {0, 128, 128, 0, 0, 0, 255},
+  {0, 128, 128, 0, 0, 0},
   /* 5% grey */
-  {32, 128, 128, 32, 32, 32, 255},
+  {32, 128, 128, 32, 32, 32},
 };
 
 
@@ -262,7 +257,6 @@ static void paint_setup_IYU2 (paintinfo * p, unsigned char *dest);
 static void paint_setup_Y41B (paintinfo * p, unsigned char *dest);
 static void paint_setup_Y42B (paintinfo * p, unsigned char *dest);
 static void paint_setup_Y800 (paintinfo * p, unsigned char *dest);
-static void paint_setup_AYUV (paintinfo * p, unsigned char *dest);
 
 #if 0
 static void paint_setup_IMC1 (paintinfo * p, unsigned char *dest);
@@ -272,10 +266,6 @@ static void paint_setup_IMC4 (paintinfo * p, unsigned char *dest);
 #endif
 static void paint_setup_YUV9 (paintinfo * p, unsigned char *dest);
 static void paint_setup_YVU9 (paintinfo * p, unsigned char *dest);
-static void paint_setup_ARGB8888 (paintinfo * p, unsigned char *dest);
-static void paint_setup_ABGR8888 (paintinfo * p, unsigned char *dest);
-static void paint_setup_RGBA8888 (paintinfo * p, unsigned char *dest);
-static void paint_setup_BGRA8888 (paintinfo * p, unsigned char *dest);
 static void paint_setup_xRGB8888 (paintinfo * p, unsigned char *dest);
 static void paint_setup_xBGR8888 (paintinfo * p, unsigned char *dest);
 static void paint_setup_RGBx8888 (paintinfo * p, unsigned char *dest);
@@ -285,15 +275,12 @@ static void paint_setup_BGR888 (paintinfo * p, unsigned char *dest);
 static void paint_setup_RGB565 (paintinfo * p, unsigned char *dest);
 static void paint_setup_xRGB1555 (paintinfo * p, unsigned char *dest);
 
-static void paint_setup_bayer (paintinfo * p, unsigned char *dest);
-
 static void paint_hline_I420 (paintinfo * p, int x, int y, int w);
 static void paint_hline_YUY2 (paintinfo * p, int x, int y, int w);
 static void paint_hline_IYU2 (paintinfo * p, int x, int y, int w);
 static void paint_hline_Y41B (paintinfo * p, int x, int y, int w);
 static void paint_hline_Y42B (paintinfo * p, int x, int y, int w);
 static void paint_hline_Y800 (paintinfo * p, int x, int y, int w);
-static void paint_hline_AYUV (paintinfo * p, int x, int y, int w);
 
 #if 0
 static void paint_hline_IMC1 (paintinfo * p, int x, int y, int w);
@@ -304,95 +291,81 @@ static void paint_hline_str3 (paintinfo * p, int x, int y, int w);
 static void paint_hline_RGB565 (paintinfo * p, int x, int y, int w);
 static void paint_hline_xRGB1555 (paintinfo * p, int x, int y, int w);
 
-static void paint_hline_bayer (paintinfo * p, int x, int y, int w);
-
 struct fourcc_list_struct fourcc_list[] = {
 /* packed */
-  {VTS_YUV, "YUY2", "YUY2", 16, paint_setup_YUY2, paint_hline_YUY2},
-  {VTS_YUV, "UYVY", "UYVY", 16, paint_setup_UYVY, paint_hline_YUY2},
-  {VTS_YUV, "Y422", "Y422", 16, paint_setup_UYVY, paint_hline_YUY2},
-  {VTS_YUV, "UYNV", "UYNV", 16, paint_setup_UYVY, paint_hline_YUY2},    /* FIXME: UYNV? */
-  {VTS_YUV, "YVYU", "YVYU", 16, paint_setup_YVYU, paint_hline_YUY2},
-  {VTS_YUV, "AYUV", "AYUV", 32, paint_setup_AYUV, paint_hline_AYUV},
+  {"YUY2", "YUY2", 16, paint_setup_YUY2, paint_hline_YUY2},
+  {"UYVY", "UYVY", 16, paint_setup_UYVY, paint_hline_YUY2},
+  {"Y422", "Y422", 16, paint_setup_UYVY, paint_hline_YUY2},
+  {"UYNV", "UYNV", 16, paint_setup_UYVY, paint_hline_YUY2},     /* FIXME: UYNV? */
+  {"YVYU", "YVYU", 16, paint_setup_YVYU, paint_hline_YUY2},
 
   /* interlaced */
-  /*{ VTS_YUV,  "IUYV", "IUY2", 16, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "IUYV", "IUY2", 16, paint_setup_YVYU, paint_hline_YUY2 }, */
 
   /* inverted */
-  /*{ VTS_YUV,  "cyuv", "cyuv", 16, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "cyuv", "cyuv", 16, paint_setup_YVYU, paint_hline_YUY2 }, */
 
-  /*{ VTS_YUV,  "Y41P", "Y41P", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "Y41P", "Y41P", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
 
   /* interlaced */
-  /*{ VTS_YUV,  "IY41", "IY41", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "IY41", "IY41", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
 
-  /*{ VTS_YUV,  "Y211", "Y211", 8, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "Y211", "Y211", 8, paint_setup_YVYU, paint_hline_YUY2 }, */
 
-  /*{ VTS_YUV,  "Y41T", "Y41T", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
-  /*{ VTS_YUV,  "Y42P", "Y42P", 16, paint_setup_YVYU, paint_hline_YUY2 }, */
-  /*{ VTS_YUV,  "CLJR", "CLJR", 8, paint_setup_YVYU, paint_hline_YUY2 }, */
-  /*{ VTS_YUV,  "IYU1", "IYU1", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
-  {VTS_YUV, "IYU2", "IYU2", 24, paint_setup_IYU2, paint_hline_IYU2},
+  /*{ "Y41T", "Y41T", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "Y42P", "Y42P", 16, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "CLJR", "CLJR", 8, paint_setup_YVYU, paint_hline_YUY2 }, */
+  /*{ "IYU1", "IYU1", 12, paint_setup_YVYU, paint_hline_YUY2 }, */
+  {"IYU2", "IYU2", 24, paint_setup_IYU2, paint_hline_IYU2},
 
 /* planar */
   /* YVU9 */
-  {VTS_YUV, "YVU9", "YVU9", 9, paint_setup_YVU9, paint_hline_YUV9},
+  {"YVU9", "YVU9", 9, paint_setup_YVU9, paint_hline_YUV9},
   /* YUV9 */
-  {VTS_YUV, "YUV9", "YUV9", 9, paint_setup_YUV9, paint_hline_YUV9},
+  {"YUV9", "YUV9", 9, paint_setup_YUV9, paint_hline_YUV9},
   /* IF09 */
   /* YV12 */
-  {VTS_YUV, "YV12", "YV12", 12, paint_setup_YV12, paint_hline_I420},
+  {"YV12", "YV12", 12, paint_setup_YV12, paint_hline_I420},
   /* I420 */
-  {VTS_YUV, "I420", "I420", 12, paint_setup_I420, paint_hline_I420},
+  {"I420", "I420", 12, paint_setup_I420, paint_hline_I420},
   /* NV12 */
   /* NV21 */
 #if 0
   /* IMC1 */
-  {VTS_YUV, "IMC1", "IMC1", 16, paint_setup_IMC1, paint_hline_IMC1},
+  {"IMC1", "IMC1", 16, paint_setup_IMC1, paint_hline_IMC1},
   /* IMC2 */
-  {VTS_YUV, "IMC2", "IMC2", 12, paint_setup_IMC2, paint_hline_IMC1},
+  {"IMC2", "IMC2", 12, paint_setup_IMC2, paint_hline_IMC1},
   /* IMC3 */
-  {VTS_YUV, "IMC3", "IMC3", 16, paint_setup_IMC3, paint_hline_IMC1},
+  {"IMC3", "IMC3", 16, paint_setup_IMC3, paint_hline_IMC1},
   /* IMC4 */
-  {VTS_YUV, "IMC4", "IMC4", 12, paint_setup_IMC4, paint_hline_IMC1},
+  {"IMC4", "IMC4", 12, paint_setup_IMC4, paint_hline_IMC1},
 #endif
   /* CLPL */
   /* Y41B */
-  {VTS_YUV, "Y41B", "Y41B", 12, paint_setup_Y41B, paint_hline_Y41B},
+  {"Y41B", "Y41B", 12, paint_setup_Y41B, paint_hline_Y41B},
   /* Y42B */
-  {VTS_YUV, "Y42B", "Y42B", 16, paint_setup_Y42B, paint_hline_Y42B},
+  {"Y42B", "Y42B", 16, paint_setup_Y42B, paint_hline_Y42B},
   /* Y800 grayscale */
-  {VTS_YUV, "Y800", "Y800", 8, paint_setup_Y800, paint_hline_Y800},
+  {"Y800", "Y800", 8, paint_setup_Y800, paint_hline_Y800},
 
-  {VTS_RGB, "RGB ", "xRGB8888", 32, paint_setup_xRGB8888, paint_hline_str4, 24,
+  {"RGB ", "xRGB8888", 32, paint_setup_xRGB8888, paint_hline_str4, 1, 24,
       0x00ff0000, 0x0000ff00, 0x000000ff},
-  {VTS_RGB, "RGB ", "xBGR8888", 32, paint_setup_xBGR8888, paint_hline_str4, 24,
+  {"RGB ", "xBGR8888", 32, paint_setup_xBGR8888, paint_hline_str4, 1, 24,
       0x000000ff, 0x0000ff00, 0x00ff0000},
-  {VTS_RGB, "RGB ", "RGBx8888", 32, paint_setup_RGBx8888, paint_hline_str4, 24,
+  {"RGB ", "RGBx8888", 32, paint_setup_RGBx8888, paint_hline_str4, 1, 24,
       0xff000000, 0x00ff0000, 0x0000ff00},
-  {VTS_RGB, "RGB ", "BGRx8888", 32, paint_setup_BGRx8888, paint_hline_str4, 24,
+  {"RGB ", "BGRx8888", 32, paint_setup_BGRx8888, paint_hline_str4, 1, 24,
       0x0000ff00, 0x00ff0000, 0xff000000},
-  {VTS_RGB, "RGB ", "ARGB8888", 32, paint_setup_ARGB8888, paint_hline_str4, 32,
-      0x00ff0000, 0x0000ff00, 0x000000ff, 0xff000000},
-  {VTS_RGB, "RGB ", "ABGR8888", 32, paint_setup_ABGR8888, paint_hline_str4, 32,
-      0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000},
-  {VTS_RGB, "RGB ", "RGBA8888", 32, paint_setup_RGBA8888, paint_hline_str4, 32,
-      0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff},
-  {VTS_RGB, "RGB ", "BGRA8888", 32, paint_setup_BGRA8888, paint_hline_str4, 32,
-      0x0000ff00, 0x00ff0000, 0xff000000, 0x000000ff},
-  {VTS_RGB, "RGB ", "RGB888", 24, paint_setup_RGB888, paint_hline_str3, 24,
+  {"RGB ", "RGB888", 24, paint_setup_RGB888, paint_hline_str3, 1, 24,
       0x00ff0000, 0x0000ff00, 0x000000ff},
-  {VTS_RGB, "RGB ", "BGR888", 24, paint_setup_BGR888, paint_hline_str3, 24,
+  {"RGB ", "BGR888", 24, paint_setup_BGR888, paint_hline_str3, 1, 24,
       0x000000ff, 0x0000ff00, 0x00ff0000},
-  {VTS_RGB, "RGB ", "RGB565", 16, paint_setup_RGB565, paint_hline_RGB565, 16,
+  {"RGB ", "RGB565", 16, paint_setup_RGB565, paint_hline_RGB565, 1, 16,
       0x0000f800, 0x000007e0, 0x0000001f},
-  {VTS_RGB, "RGB ", "xRGB1555", 16, paint_setup_xRGB1555, paint_hline_xRGB1555,
-        15,
+  {"RGB ", "xRGB1555", 16, paint_setup_xRGB1555, paint_hline_xRGB1555, 1, 15,
       0x00007c00, 0x000003e0, 0x0000001f},
-
-  {VTS_BAYER, "BAY8", "Bayer", 8, paint_setup_bayer, paint_hline_bayer}
 };
-int n_fourccs = G_N_ELEMENTS (fourcc_list);
+int n_fourccs = sizeof (fourcc_list) / sizeof (fourcc_list[0]);
 
 struct fourcc_list_struct *
 paintinfo_find_by_structure (const GstStructure * structure)
@@ -415,7 +388,7 @@ paintinfo_find_by_structure (const GstStructure * structure)
       s = fourcc_list[i].fourcc;
       /* g_print("testing %" GST_FOURCC_FORMAT " and %s\n", GST_FOURCC_ARGS(format), s); */
       fourcc = GST_MAKE_FOURCC (s[0], s[1], s[2], s[3]);
-      if (fourcc_list[i].type == VTS_YUV && fourcc == format) {
+      if (fourcc == format) {
         return fourcc_list + i;
       }
     }
@@ -423,7 +396,6 @@ paintinfo_find_by_structure (const GstStructure * structure)
     int red_mask;
     int green_mask;
     int blue_mask;
-    int alpha_mask;
     int depth;
     int bpp;
 
@@ -433,33 +405,14 @@ paintinfo_find_by_structure (const GstStructure * structure)
     ret &= gst_structure_get_int (structure, "depth", &depth);
     ret &= gst_structure_get_int (structure, "bpp", &bpp);
 
-    if (depth == 32) {
-      ret &= gst_structure_get_int (structure, "alpha_mask", &alpha_mask);
-      ret &= (alpha_mask != 0);
-    } else {
-      alpha_mask = 0;
-    }
-
-    if (!ret) {
-      GST_WARNING ("incomplete caps structure: %" GST_PTR_FORMAT, structure);
-      return NULL;
-    }
-
     for (i = 0; i < n_fourccs; i++) {
-      if (fourcc_list[i].type == VTS_RGB &&
+      if (strcmp (fourcc_list[i].fourcc, "RGB ") == 0 &&
           fourcc_list[i].red_mask == red_mask &&
           fourcc_list[i].green_mask == green_mask &&
           fourcc_list[i].blue_mask == blue_mask &&
-          (alpha_mask == 0 || fourcc_list[i].alpha_mask == alpha_mask) &&
           fourcc_list[i].depth == depth && fourcc_list[i].bitspp == bpp) {
         return fourcc_list + i;
-      }
-    }
-    return NULL;
-  } else if (strcmp (media_type, "video/x-raw-bayer") == 0) {
-    for (i = 0; i < n_fourccs; i++) {
-      if (fourcc_list[i].type == VTS_BAYER) {
-        return fourcc_list + i;
+
       }
     }
     return NULL;
@@ -483,7 +436,7 @@ paintrect_find_fourcc (int find_fourcc)
     fourcc = GST_MAKE_FOURCC (s[0], s[1], s[2], s[3]);
     if (find_fourcc == fourcc) {
       /* If YUV format, it's good */
-      if (!fourcc_list[i].type == VTS_YUV) {
+      if (!fourcc_list[i].ext_caps) {
         return fourcc_list + i;
       }
 
@@ -510,9 +463,7 @@ paintrect_find_name (const char *name)
 GstStructure *
 paint_get_structure (struct fourcc_list_struct * format)
 {
-  GstStructure *structure = NULL;
   unsigned int fourcc;
-  int endianness;
 
   g_return_val_if_fail (format, NULL);
 
@@ -520,37 +471,25 @@ paint_get_structure (struct fourcc_list_struct * format)
       GST_MAKE_FOURCC (format->fourcc[0], format->fourcc[1], format->fourcc[2],
       format->fourcc[3]);
 
-  switch (format->type) {
-    case VTS_RGB:
-      if (format->bitspp == 16) {
-        endianness = G_BYTE_ORDER;
-      } else {
-        endianness = G_BIG_ENDIAN;
-      }
-      structure = gst_structure_new ("video/x-raw-rgb",
-          "bpp", G_TYPE_INT, format->bitspp,
-          "endianness", G_TYPE_INT, endianness,
-          "depth", G_TYPE_INT, format->depth,
-          "red_mask", G_TYPE_INT, format->red_mask,
-          "green_mask", G_TYPE_INT, format->green_mask,
-          "blue_mask", G_TYPE_INT, format->blue_mask, NULL);
-      if (format->depth == 32 && format->alpha_mask > 0) {
-        gst_structure_set (structure, "alpha_mask", G_TYPE_INT,
-            format->alpha_mask, NULL);
-      }
-      break;
-    case VTS_YUV:
-      structure = gst_structure_new ("video/x-raw-yuv",
-          "format", GST_TYPE_FOURCC, fourcc, NULL);
-      break;
-    case VTS_BAYER:
-      structure = gst_structure_new ("video/x-raw-bayer", NULL);
-      break;
-    default:
-      g_assert_not_reached ();
-      break;
+  if (format->ext_caps) {
+    int endianness;
+
+    if (format->bitspp == 16) {
+      endianness = G_BYTE_ORDER;
+    } else {
+      endianness = G_BIG_ENDIAN;
+    }
+    return gst_structure_new ("video/x-raw-rgb",
+        "bpp", G_TYPE_INT, format->bitspp,
+        "endianness", G_TYPE_INT, endianness,
+        "depth", G_TYPE_INT, format->depth,
+        "red_mask", G_TYPE_INT, format->red_mask,
+        "green_mask", G_TYPE_INT, format->green_mask,
+        "blue_mask", G_TYPE_INT, format->blue_mask, NULL);
+  } else {
+    return gst_structure_new ("video/x-raw-yuv",
+        "format", GST_TYPE_FOURCC, fourcc, NULL);
   }
-  return structure;
 }
 
 /* returns the size in bytes for one video frame of the given dimensions
@@ -558,7 +497,7 @@ paint_get_structure (struct fourcc_list_struct * format)
 int
 gst_video_test_src_get_size (GstVideoTestSrc * v, int w, int h)
 {
-  paintinfo pi = { NULL, };
+  paintinfo pi = { 0 };
   paintinfo *p = &pi;
   struct fourcc_list_struct *fourcc;
 
@@ -580,7 +519,7 @@ gst_video_test_src_smpte (GstVideoTestSrc * v, unsigned char *dest, int w,
   int i;
   int y1, y2;
   int j;
-  paintinfo pi = { NULL, };
+  paintinfo pi;
   paintinfo *p = &pi;
   struct fourcc_list_struct *fourcc;
 
@@ -688,7 +627,7 @@ gst_video_test_src_snow (GstVideoTestSrc * v, unsigned char *dest, int w, int h)
 {
   int i;
   int j;
-  paintinfo pi = { NULL, };
+  paintinfo pi;
   paintinfo *p = &pi;
   struct fourcc_list_struct *fourcc;
   struct vts_color_struct color;
@@ -717,12 +656,12 @@ gst_video_test_src_snow (GstVideoTestSrc * v, unsigned char *dest, int w, int h)
   }
 }
 
-static void
-gst_video_test_src_unicolor (GstVideoTestSrc * v, unsigned char *dest, int w,
-    int h, const struct vts_color_struct *color)
+void
+gst_video_test_src_black (GstVideoTestSrc * v, unsigned char *dest, int w,
+    int h)
 {
   int i;
-  paintinfo pi = { NULL, };
+  paintinfo pi;
   paintinfo *p = &pi;
   struct fourcc_list_struct *fourcc;
 
@@ -735,299 +674,33 @@ gst_video_test_src_unicolor (GstVideoTestSrc * v, unsigned char *dest, int w,
   fourcc->paint_setup (p, dest);
   p->paint_hline = fourcc->paint_hline;
 
-  p->color = color;
+  p->color = vts_colors + COLOR_BLACK;
 
   for (i = 0; i < h; i++) {
     p->paint_hline (p, 0, i, w);
   }
 }
 
-void
-gst_video_test_src_black (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  gst_video_test_src_unicolor (v, dest, w, h, vts_colors + COLOR_BLACK);
-}
-
-void
-gst_video_test_src_white (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  gst_video_test_src_unicolor (v, dest, w, h, vts_colors + COLOR_WHITE);
-}
-
-void
-gst_video_test_src_red (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  gst_video_test_src_unicolor (v, dest, w, h, vts_colors + COLOR_RED);
-}
-
-void
-gst_video_test_src_green (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  gst_video_test_src_unicolor (v, dest, w, h, vts_colors + COLOR_GREEN);
-}
-
-void
-gst_video_test_src_blue (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  gst_video_test_src_unicolor (v, dest, w, h, vts_colors + COLOR_BLUE);
-}
-
-void
-gst_video_test_src_checkers1 (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  int x, y;
-  paintinfo pi = { NULL, };
-  paintinfo *p = &pi;
-  struct fourcc_list_struct *fourcc;
-
-  p->width = w;
-  p->height = h;
-  fourcc = v->fourcc;
-  if (fourcc == NULL)
-    return;
-
-  fourcc->paint_setup (p, dest);
-  p->paint_hline = fourcc->paint_hline;
-
-  for (y = 0; y < h; y++) {
-    p->color = vts_colors + COLOR_GREEN;
-    p->paint_hline (p, 0, y, w);
-    for (x = (y % 2); x < w; x += 2) {
-      p->color = vts_colors + COLOR_RED;
-      p->paint_hline (p, x, y, 1);
-    }
-  }
-}
-
-void
-gst_video_test_src_checkers2 (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  int x, y;
-  paintinfo pi = { NULL, };
-  paintinfo *p = &pi;
-  struct fourcc_list_struct *fourcc;
-
-  p->width = w;
-  p->height = h;
-  fourcc = v->fourcc;
-  if (fourcc == NULL)
-    return;
-
-  fourcc->paint_setup (p, dest);
-  p->paint_hline = fourcc->paint_hline;
-
-  p->color = vts_colors + COLOR_GREEN;
-  for (y = 0; y < h; y++) {
-    p->paint_hline (p, 0, y, w);
-  }
-
-  for (y = 0; y < h; y += 2) {
-    for (x = ((y % 4) == 0) ? 0 : 2; x < w; x += 4) {
-      guint len = (x < (w - 1)) ? 2 : (w - x);
-
-      p->color = vts_colors + COLOR_RED;
-      p->paint_hline (p, x, y + 0, len);
-      if (G_LIKELY ((y + 1) < h)) {
-        p->paint_hline (p, x, y + 1, len);
-      }
-    }
-  }
-}
-
-void
-gst_video_test_src_checkers4 (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  int x, y;
-  paintinfo pi = { NULL, };
-  paintinfo *p = &pi;
-  struct fourcc_list_struct *fourcc;
-
-  p->width = w;
-  p->height = h;
-  fourcc = v->fourcc;
-  if (fourcc == NULL)
-    return;
-
-  fourcc->paint_setup (p, dest);
-  p->paint_hline = fourcc->paint_hline;
-
-  p->color = vts_colors + COLOR_GREEN;
-  for (y = 0; y < h; y++) {
-    p->paint_hline (p, 0, y, w);
-  }
-
-  for (y = 0; y < h; y += 4) {
-    for (x = ((y % 8) == 0) ? 0 : 4; x < w; x += 8) {
-      guint len = (x < (w - 3)) ? 4 : (w - x);
-
-      p->color = vts_colors + COLOR_RED;
-      p->paint_hline (p, x, y + 0, len);
-      if (G_LIKELY ((y + 1) < h)) {
-        p->paint_hline (p, x, y + 1, len);
-        if (G_LIKELY ((y + 2) < h)) {
-          p->paint_hline (p, x, y + 2, len);
-          if (G_LIKELY ((y + 3) < h)) {
-            p->paint_hline (p, x, y + 3, len);
-          }
-        }
-      }
-    }
-  }
-}
-
-void
-gst_video_test_src_checkers8 (GstVideoTestSrc * v, guchar * dest, int w, int h)
-{
-  int x, y;
-  paintinfo pi = { NULL, };
-  paintinfo *p = &pi;
-  struct fourcc_list_struct *fourcc;
-
-  p->width = w;
-  p->height = h;
-  fourcc = v->fourcc;
-  if (fourcc == NULL)
-    return;
-
-  fourcc->paint_setup (p, dest);
-  p->paint_hline = fourcc->paint_hline;
-
-  p->color = vts_colors + COLOR_GREEN;
-  for (y = 0; y < h; y++) {
-    p->paint_hline (p, 0, y, w);
-  }
-
-  for (y = 0; y < h; y += 8) {
-    for (x = ((GST_ROUND_UP_8 (y) % 16) == 0) ? 0 : 8; x < w; x += 16) {
-      guint len = (x < (w - 7)) ? 8 : (w - x);
-
-      p->color = vts_colors + COLOR_RED;
-      p->paint_hline (p, x, y + 0, len);
-      if (G_LIKELY ((y + 1) < h)) {
-        p->paint_hline (p, x, y + 1, len);
-        if (G_LIKELY ((y + 2) < h)) {
-          p->paint_hline (p, x, y + 2, len);
-          if (G_LIKELY ((y + 3) < h)) {
-            p->paint_hline (p, x, y + 3, len);
-            if (G_LIKELY ((y + 4) < h)) {
-              p->paint_hline (p, x, y + 4, len);
-              if (G_LIKELY ((y + 5) < h)) {
-                p->paint_hline (p, x, y + 5, len);
-                if (G_LIKELY ((y + 6) < h)) {
-                  p->paint_hline (p, x, y + 6, len);
-                  if (G_LIKELY ((y + 7) < h)) {
-                    p->paint_hline (p, x, y + 7, len);
-                  }
-                }
-              }
-            }
-          }
-        }
-      }
-    }
-  }
-}
-
-#undef SCALE_AMPLITUDE
-void
-gst_video_test_src_circular (GstVideoTestSrc * v, unsigned char *dest,
-    int w, int h)
-{
-  int i;
-  int j;
-  paintinfo pi = { NULL, };
-  paintinfo *p = &pi;
-  struct fourcc_list_struct *fourcc;
-  struct vts_color_struct color;
-  static uint8_t sine_array[256];
-  static int sine_array_inited = FALSE;
-  double freq[8];
-
-#ifdef SCALE_AMPLITUDE
-  double ampl[8];
-#endif
-  int d;
-
-  if (!sine_array_inited) {
-    for (i = 0; i < 256; i++) {
-      sine_array[i] =
-          floor (255 * (0.5 + 0.5 * sin (i * 2 * M_PI / 256)) + 0.5);
-    }
-    sine_array_inited = TRUE;
-  }
-
-  p->width = w;
-  p->height = h;
-  fourcc = v->fourcc;
-  if (fourcc == NULL)
-    return;
-
-  fourcc->paint_setup (p, dest);
-  p->paint_hline = fourcc->paint_hline;
-
-  color = vts_colors[COLOR_BLACK];
-  p->color = &color;
-
-  for (i = 1; i < 8; i++) {
-    freq[i] = 200 * pow (2.0, -(i - 1) / 4.0);
-#ifdef SCALE_AMPLITUDE
-    {
-      double x;
-
-      x = 2 * M_PI * freq[i] / w;
-      ampl[i] = sin (x) / x;
-    }
-#endif
-  }
-
-  for (i = 0; i < w; i++) {
-    for (j = 0; j < h; j++) {
-      double dist;
-      int seg;
-
-      dist =
-          sqrt ((2 * i - w) * (2 * i - w) + (2 * j - h) * (2 * j -
-              h)) / (2 * w);
-      seg = floor (dist * 16);
-      if (seg == 0 || seg >= 8) {
-        color.Y = 255;
-      } else {
-#ifdef SCALE_AMPLITUDE
-        double a;
-#endif
-        d = floor (256 * dist * freq[seg] + 0.5);
-#ifdef SCALE_AMPLITUDE
-        a = ampl[seg];
-        if (a < 0)
-          a = 0;
-        color.Y = 128 + a * (sine_array[d & 0xff] - 128);
-#else
-        color.Y = sine_array[d & 0xff];
-#endif
-      }
-      color.R = color.Y;
-      color.G = color.Y;
-      color.B = color.Y;
-      p->paint_hline (p, i, j, 1);
-    }
-  }
-}
+#define ROUND_UP_2(x)  (((x)+1)&~1)
+#define ROUND_UP_4(x)  (((x)+3)&~3)
+#define ROUND_UP_8(x)  (((x)+7)&~7)
 
 static void
 paint_setup_I420 (paintinfo * p, unsigned char *dest)
 {
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
-  p->up = p->yp + p->ystride * GST_ROUND_UP_2 (p->height);
-  p->ustride = GST_ROUND_UP_8 (p->width) / 2;
-  p->vp = p->up + p->ustride * GST_ROUND_UP_2 (p->height) / 2;
-  p->vstride = GST_ROUND_UP_8 (p->ystride) / 2;
-  p->endptr = p->vp + p->vstride * GST_ROUND_UP_2 (p->height) / 2;
+  p->ystride = ROUND_UP_4 (p->width);
+  p->up = p->yp + p->ystride * ROUND_UP_2 (p->height);
+  p->ustride = ROUND_UP_8 (p->width) / 2;
+  p->vp = p->up + p->ustride * ROUND_UP_2 (p->height) / 2;
+  p->vstride = ROUND_UP_8 (p->ystride) / 2;
+  p->endptr = p->vp + p->vstride * ROUND_UP_2 (p->height) / 2;
 }
 
 static void
 paint_hline_I420 (paintinfo * p, int x, int y, int w)
 {
+#ifdef oil_splat_u8_ns
   int x1 = x / 2;
   int x2 = (x + w) / 2;
   int offset = y * p->ystride;
@@ -1036,29 +709,19 @@ paint_hline_I420 (paintinfo * p, int x, int y, int w)
   oil_splat_u8_ns (p->yp + offset + x, &p->color->Y, w);
   oil_splat_u8_ns (p->up + offset1 + x1, &p->color->U, x2 - x1);
   oil_splat_u8_ns (p->vp + offset1 + x1, &p->color->V, x2 - x1);
+#endif
 }
 
 static void
 paint_setup_YV12 (paintinfo * p, unsigned char *dest)
 {
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
-  p->vp = p->yp + p->ystride * GST_ROUND_UP_2 (p->height);
-  p->vstride = GST_ROUND_UP_8 (p->ystride) / 2;
-  p->up = p->vp + p->vstride * GST_ROUND_UP_2 (p->height) / 2;
-  p->ustride = GST_ROUND_UP_8 (p->ystride) / 2;
-  p->endptr = p->up + p->ustride * GST_ROUND_UP_2 (p->height) / 2;
-}
-
-static void
-paint_setup_AYUV (paintinfo * p, unsigned char *dest)
-{
-  p->ap = dest;
-  p->yp = dest + 1;
-  p->up = dest + 2;
-  p->vp = dest + 3;
-  p->ystride = p->width * 4;
-  p->endptr = dest + p->ystride * p->height;
+  p->ystride = ROUND_UP_4 (p->width);
+  p->vp = p->yp + p->ystride * ROUND_UP_2 (p->height);
+  p->vstride = ROUND_UP_8 (p->ystride) / 2;
+  p->up = p->vp + p->vstride * ROUND_UP_2 (p->height) / 2;
+  p->ustride = ROUND_UP_8 (p->ystride) / 2;
+  p->endptr = p->up + p->ustride * ROUND_UP_2 (p->height) / 2;
 }
 
 static void
@@ -1067,7 +730,7 @@ paint_setup_YUY2 (paintinfo * p, unsigned char *dest)
   p->yp = dest;
   p->up = dest + 1;
   p->vp = dest + 3;
-  p->ystride = GST_ROUND_UP_2 (p->width) * 2;
+  p->ystride = ROUND_UP_2 (p->width) * 2;
   p->endptr = dest + p->ystride * p->height;
 }
 
@@ -1077,7 +740,7 @@ paint_setup_UYVY (paintinfo * p, unsigned char *dest)
   p->yp = dest + 1;
   p->up = dest;
   p->vp = dest + 2;
-  p->ystride = GST_ROUND_UP_2 (p->width) * 2;
+  p->ystride = ROUND_UP_2 (p->width) * 2;
   p->endptr = dest + p->ystride * p->height;
 }
 
@@ -1087,22 +750,10 @@ paint_setup_YVYU (paintinfo * p, unsigned char *dest)
   p->yp = dest;
   p->up = dest + 3;
   p->vp = dest + 1;
-  p->ystride = GST_ROUND_UP_2 (p->width) * 2;
+  p->ystride = ROUND_UP_2 (p->width * 2);
   p->endptr = dest + p->ystride * p->height;
 }
 
-static void
-paint_hline_AYUV (paintinfo * p, int x, int y, int w)
-{
-  int offset;
-
-  offset = (y * p->ystride) + (x * 4);
-  oil_splat_u8 (p->yp + offset, 4, &p->color->Y, w);
-  oil_splat_u8 (p->up + offset, 4, &p->color->U, w);
-  oil_splat_u8 (p->vp + offset, 4, &p->color->V, w);
-  oil_splat_u8 (p->ap + offset, 4, &p->color->A, w);
-}
-
 static void
 paint_hline_YUY2 (paintinfo * p, int x, int y, int w)
 {
@@ -1123,7 +774,7 @@ paint_setup_IYU2 (paintinfo * p, unsigned char *dest)
   p->yp = dest + 1;
   p->up = dest + 0;
   p->vp = dest + 2;
-  p->ystride = GST_ROUND_UP_4 (p->width * 3);
+  p->ystride = ROUND_UP_4 (p->width * 3);
   p->endptr = dest + p->ystride * p->height;
 }
 
@@ -1142,17 +793,18 @@ static void
 paint_setup_Y41B (paintinfo * p, unsigned char *dest)
 {
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
+  p->ystride = ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * p->height;
-  p->ustride = GST_ROUND_UP_8 (p->width) / 4;
+  p->ustride = ROUND_UP_8 (p->width) / 4;
   p->vp = p->up + p->ustride * p->height;
-  p->vstride = GST_ROUND_UP_8 (p->width) / 4;
+  p->vstride = ROUND_UP_8 (p->width) / 4;
   p->endptr = p->vp + p->vstride * p->height;
 }
 
 static void
 paint_hline_Y41B (paintinfo * p, int x, int y, int w)
 {
+#ifdef oil_splat_u8_ns
   int x1 = x / 4;
   int x2 = (x + w) / 4;
   int offset = y * p->ystride;
@@ -1161,23 +813,25 @@ paint_hline_Y41B (paintinfo * p, int x, int y, int w)
   oil_splat_u8_ns (p->yp + offset + x, &p->color->Y, w);
   oil_splat_u8_ns (p->up + offset1 + x1, &p->color->U, x2 - x1);
   oil_splat_u8_ns (p->vp + offset1 + x1, &p->color->V, x2 - x1);
+#endif
 }
 
 static void
 paint_setup_Y42B (paintinfo * p, unsigned char *dest)
 {
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
+  p->ystride = ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * p->height;
-  p->ustride = GST_ROUND_UP_8 (p->width) / 2;
+  p->ustride = ROUND_UP_8 (p->width) / 2;
   p->vp = p->up + p->ustride * p->height;
-  p->vstride = GST_ROUND_UP_8 (p->width) / 2;
+  p->vstride = ROUND_UP_8 (p->width) / 2;
   p->endptr = p->vp + p->vstride * p->height;
 }
 
 static void
 paint_hline_Y42B (paintinfo * p, int x, int y, int w)
 {
+#ifdef oil_splat_u8_ns
   int x1 = x / 2;
   int x2 = (x + w) / 2;
   int offset = y * p->ystride;
@@ -1186,6 +840,7 @@ paint_hline_Y42B (paintinfo * p, int x, int y, int w)
   oil_splat_u8_ns (p->yp + offset + x, &p->color->Y, w);
   oil_splat_u8_ns (p->up + offset1 + x1, &p->color->U, x2 - x1);
   oil_splat_u8_ns (p->vp + offset1 + x1, &p->color->V, x2 - x1);
+#endif
 }
 
 static void
@@ -1193,16 +848,18 @@ paint_setup_Y800 (paintinfo * p, unsigned char *dest)
 {
   /* untested */
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
+  p->ystride = ROUND_UP_4 (p->width);
   p->endptr = dest + p->ystride * p->height;
 }
 
 static void
 paint_hline_Y800 (paintinfo * p, int x, int y, int w)
 {
+#ifdef oil_splat_u8_ns
   int offset = y * p->ystride;
 
   oil_splat_u8_ns (p->yp + offset + x, &p->color->Y, w);
+#endif
 }
 
 #if 0
@@ -1255,35 +912,36 @@ paint_hline_IMC1 (paintinfo * p, int x, int y, int w)
 static void
 paint_setup_YVU9 (paintinfo * p, unsigned char *dest)
 {
-  int h = GST_ROUND_UP_4 (p->height);
+  int h = ROUND_UP_4 (p->height);
 
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
-  p->vp = p->yp + p->ystride * GST_ROUND_UP_4 (p->height);
-  p->vstride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->up = p->vp + p->vstride * GST_ROUND_UP_4 (h / 4);
-  p->ustride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->endptr = p->up + p->ustride * GST_ROUND_UP_4 (h / 4);
+  p->ystride = ROUND_UP_4 (p->width);
+  p->vp = p->yp + p->ystride * ROUND_UP_4 (p->height);
+  p->vstride = ROUND_UP_4 (p->ystride / 4);
+  p->up = p->vp + p->vstride * ROUND_UP_4 (h / 4);
+  p->ustride = ROUND_UP_4 (p->ystride / 4);
+  p->endptr = p->up + p->ustride * ROUND_UP_4 (h / 4);
 }
 
 static void
 paint_setup_YUV9 (paintinfo * p, unsigned char *dest)
 {
   /* untested */
-  int h = GST_ROUND_UP_4 (p->height);
+  int h = ROUND_UP_4 (p->height);
 
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
+  p->ystride = ROUND_UP_4 (p->width);
   p->up = p->yp + p->ystride * h;
-  p->ustride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->vp = p->up + p->ustride * GST_ROUND_UP_4 (h / 4);
-  p->vstride = GST_ROUND_UP_4 (p->ystride / 4);
-  p->endptr = p->vp + p->vstride * GST_ROUND_UP_4 (h / 4);
+  p->ustride = ROUND_UP_4 (p->ystride / 4);
+  p->vp = p->up + p->ustride * ROUND_UP_4 (h / 4);
+  p->vstride = ROUND_UP_4 (p->ystride / 4);
+  p->endptr = p->vp + p->vstride * ROUND_UP_4 (h / 4);
 }
 
 static void
 paint_hline_YUV9 (paintinfo * p, int x, int y, int w)
 {
+#ifdef oil_splat_u8_ns
   int x1 = x / 4;
   int x2 = (x + w) / 4;
   int offset = y * p->ystride;
@@ -1292,34 +950,7 @@ paint_hline_YUV9 (paintinfo * p, int x, int y, int w)
   oil_splat_u8_ns (p->yp + offset + x, &p->color->Y, w);
   oil_splat_u8_ns (p->up + offset1 + x1, &p->color->U, x2 - x1);
   oil_splat_u8_ns (p->vp + offset1 + x1, &p->color->V, x2 - x1);
-}
-
-static void
-paint_setup_ARGB8888 (paintinfo * p, unsigned char *dest)
-{
-  paint_setup_xRGB8888 (p, dest);
-  p->ap = dest;
-}
-
-static void
-paint_setup_ABGR8888 (paintinfo * p, unsigned char *dest)
-{
-  paint_setup_xBGR8888 (p, dest);
-  p->ap = dest;
-}
-
-static void
-paint_setup_RGBA8888 (paintinfo * p, unsigned char *dest)
-{
-  paint_setup_RGBx8888 (p, dest);
-  p->ap = dest + 3;
-}
-
-static void
-paint_setup_BGRA8888 (paintinfo * p, unsigned char *dest)
-{
-  paint_setup_BGRx8888 (p, dest);
-  p->ap = dest + 3;
+#endif
 }
 
 static void
@@ -1368,7 +999,7 @@ paint_setup_RGB888 (paintinfo * p, unsigned char *dest)
   p->yp = dest + 0;
   p->up = dest + 1;
   p->vp = dest + 2;
-  p->ystride = GST_ROUND_UP_4 (p->width * 3);
+  p->ystride = ROUND_UP_4 (p->width * 3);
   p->endptr = p->dest + p->ystride * p->height;
 }
 
@@ -1378,7 +1009,7 @@ paint_setup_BGR888 (paintinfo * p, unsigned char *dest)
   p->yp = dest + 2;
   p->up = dest + 1;
   p->vp = dest + 0;
-  p->ystride = GST_ROUND_UP_4 (p->width * 3);
+  p->ystride = ROUND_UP_4 (p->width * 3);
   p->endptr = p->dest + p->ystride * p->height;
 }
 
@@ -1390,10 +1021,6 @@ paint_hline_str4 (paintinfo * p, int x, int y, int w)
   oil_splat_u8 (p->yp + offset + x * 4, 4, &p->color->R, w);
   oil_splat_u8 (p->up + offset + x * 4, 4, &p->color->G, w);
   oil_splat_u8 (p->vp + offset + x * 4, 4, &p->color->B, w);
-
-  if (p->ap != NULL) {
-    oil_splat_u8 (p->ap + offset + (x * 4), 4, &p->color->A, w);
-  }
 }
 
 static void
@@ -1410,7 +1037,7 @@ static void
 paint_setup_RGB565 (paintinfo * p, unsigned char *dest)
 {
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width * 2);
+  p->ystride = ROUND_UP_4 (p->width * 2);
   p->endptr = p->dest + p->ystride * p->height;
 }
 
@@ -1436,7 +1063,7 @@ static void
 paint_setup_xRGB1555 (paintinfo * p, unsigned char *dest)
 {
   p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width * 2);
+  p->ystride = ROUND_UP_4 (p->width * 2);
   p->endptr = p->dest + p->ystride * p->height;
 }
 
@@ -1457,38 +1084,3 @@ paint_hline_xRGB1555 (paintinfo * p, int x, int y, int w)
   oil_splat_u8 (p->yp + offset + x * 2 + 1, 2, &b, w);
 #endif
 }
-
-
-static void
-paint_setup_bayer (paintinfo * p, unsigned char *dest)
-{
-  p->yp = dest;
-  p->ystride = GST_ROUND_UP_4 (p->width);
-  p->endptr = p->dest + p->ystride * p->height;
-}
-
-static void
-paint_hline_bayer (paintinfo * p, int x, int y, int w)
-{
-  int offset = y * p->ystride;
-  uint8_t *dest = p->yp + offset;
-  int i;
-
-  if (y & 1) {
-    for (i = x; i < x + w; i++) {
-      if (i & 1) {
-        dest[i] = p->color->G;
-      } else {
-        dest[i] = p->color->B;
-      }
-    }
-  } else {
-    for (i = x; i < x + w; i++) {
-      if (i & 1) {
-        dest[i] = p->color->R;
-      } else {
-        dest[i] = p->color->G;
-      }
-    }
-  }
-}
